--Thanks to vhaarr for helping me out with reducing the length of code
--Thanks to pastamancer for fixing the issues with Supreme Power Flasks and pointing in right direction for others
--Window Resizing code based off the dragbar from violation
--And also thanks to all those in #wowace for the various suggestions

BigBrother = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")

local addon = BigBrother

local AceEvent = AceLibrary("AceEvent-2.0")
local RL = AceLibrary("Roster-2.1")
local aura = AceLibrary("SpecialEvents-Aura-2.0")
local parser = ParserLib:GetInstance("1.1");

local buffs = {
	"Flask of Arcane Fortification",
	"Chromatic Resistance",
	"Distilled Wisdom",
	"Flask of Fortification",
	"Flask of Mighty Restoration",
	"Flask of Relentless Assault",
	"Flask of Shadow Fortification",
	"Supreme Power",
	"Flask of the Titans",
}

local color = "|cffff8040%s|r"
local MissingBuffList={}

BigBrotherVars={}

local options = {
	type = 'group',
	handler = BigBrother,
	args = {
		flaskcheck = {
			name = "Flask check",
			desc = "Checks for flasks on the raid.",
			type = 'group',
			args = {
				self = {
					name = "Self",
					desc = "Reports result only to yourself.",
					type = 'execute',
					func = "FlaskCheck",
					passValue = "SELF",
				},
				party = {
					name = "Party",
					desc = "Reports result to your party.",
					type = 'execute',
					func = "FlaskCheck",
					disabled = function() return GetNumPartyMembers()==0 end,
					passValue = "PARTY",
				},
				raid = {
					name = "Raid",
					desc = "Reports result to your raid.",
					type = 'execute',
					func = "FlaskCheck",
					disabled = function() return GetNumRaidMembers()==0 end,
					passValue = "RAID",
				},
				guild = {
					name = "Guild",
					desc = "Reports result to guild chat.",
					type = 'execute',
					func = "FlaskCheck",
					passValue = "GUILD",
				},
				officer = {
					name = "Officer",
					desc = "Reports result to officer chat.",
					type = 'execute',
					func = "FlaskCheck",
					passValue = "OFFICER",
				}
			}
			
		},
		polymorph = {
			name  = "Polymorph",
			desc = "Reports if hostile polymorphs are broken and by who.",
			type = 'toggle',
			get = function() return BigBrotherVars.PolyBreak end,
			set = function(v)
				BigBrotherVars.PolyBreak=v 
				if v then
					parser:RegisterInfoType("BigBrother","dispel", addon.SheepBreaker)
				else
					parser:UnregisterInfoType("BigBrother","dispel")
				end
			end,
			map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
		},
		misdirect = {
			name  = "Misdirect",
			desc = "Reports who gains misdirection.",
			type = 'toggle',
			get = function() return BigBrotherVars.Misdirect end,
			set = function(v) BigBrotherVars.Misdirect = v end,
			map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
		},
		buffcheck = {
			name = "BuffCheck",
			desc = "Pops up a window to check various raid/elixir buffs (drag the bottom to resize).",
			type = 'execute',
			func = function() CreateBuffWindow() end,
		}
	}
}

function addon:OnInitialize()
	self:RegisterChatCommand("/bb", "/bigbrother", options, "BIGBROTHER")
end

function addon:OnEnable()
	parser:RegisterInfoType("BigBrother","dispel", addon.SheepBreaker)
	parser:RegisterInfoType("BigBrother_BuffWindow","dispel", addon.BuffUpdating)
	self:RegisterEvent("SpecialEvents_UnitBuffGained")
	self:RegisterEvent("SpecialEvents_UnitBuffLost", "BuffUpdating")
	BigBrotherVars.PolyBreak = true
	BigBrotherVars.Misdirect = true
end

function addon:SpecialEvents_UnitBuffGained(unit, name, index, count, icon, rank)
	if name == "Misdirection" then
		if BigBrotherVars.Misdirect then
			self:Print("Misdirection on |cff40ff40"..UnitName(unit).."|r.")
		end
	else
		self:BuffUpdating()
	end
end

function addon:SendMessageList(Pre,List,Where)
	if #List > 0 then
		if Where == "SELF" then
			self:Print(string.format(color, Pre..":") .. " " .. table.concat(List, ", "))
		else
			SendChatMessage(Pre..": "..table.concat(List, ", "),Where)
		end
	end
end

function addon:HasBuff(player)
	for k, v in pairs(MissingBuffList) do
		if v==player then
			table.remove(MissingBuffList,k)
		end
	end
end

function addon:FlaskCheck(Where)
	local totalCount = 0

	MissingBuffList={}
	for unit in RL:IterateRoster(false) do
		table.insert(MissingBuffList,unit.name)
	end

	for i, v in ipairs(buffs) do
		local t = self:BuffPlayerList(v)
		self:SendMessageList(v, t, Where)
		totalCount = totalCount + #t
	end
	
	self:SendMessageList("No Flask", MissingBuffList, Where)
end

function addon:BuffPlayerList(buffname)
	local list = {}
	for unit in RL:IterateRoster(false) do
		if aura:UnitHasBuff(unit.unitid, buffname) then
			table.insert(list, unit.name)
			self:HasBuff(unit.name)
		end
	end
	return list
end

function addon:SheepBreaker(info)
	--Since we don't register the event for when variables are loaded
	if not BigBrotherVars.PolyBreak then
		parser:UnregisterInfoType("BigBrother","dispel")
		return
	end

	local Polymorph=string.find(info.skill,"Polymorph")
	if Polymorph and info.source then
		local SourceHostile,VictimHostile

		if info.source==ParserLib_SELF then
			info.source=UnitName("PLAYER")
		end
		if info.victim==ParserLib_SELF then
			info.victim=UnitName("PLAYER")
		end

		if RL:GetUnitIDFromName(info.victim) then
			--We don't care about friendlies being polymorphed
			return
		else
			VictimHostile="|cffff4040"
		end

		if RL:GetUnitIDFromName(info.source) then
			SourceHostile="|cff40ff40"
		else
			SourceHostile="|cffff4040"
		end

		if info.sourceSkill then
			BigBrother:Print("Polymorph on "..VictimHostile..info.victim.."|r removed by "..SourceHostile..info.source.."|r's "..info.sourceSkill)
		else
			BigBrother:Print("Polymorph on "..VictimHostile..info.victim.."|r removed by "..SourceHostile..info.source.."|r")
		end
	elseif Polymorph then
		if info.victim==ParserLib_SELF then
			info.victim=UnitName("PLAYER")
		end

		if RL:GetUnitIDFromName(info.victim) then
			--We don't care about friendlies being polymorphed
			return
		else
			VictimHostile="|cffff4040"
		end

		BigBrother:Print("Polymorph on "..VictimHostile..info.victim.."|r is removed")
	end
end

function addon:BuffUpdating(info)
	if BigBrother_BuffWindow and BigBrother_BuffWindow:IsShown() then
		addon:ScheduleEvent(BuffWindow_Update,0.25)
	end
end

